#
# Copyright 2019 David Carter <dcarter@davesrocketshop.com>
# 
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

#Author-David Carter

import adsk.core, adsk.fusion, adsk.cam
from .NoseShapeHandler import NoseShapeHandler
    
class ConeShapeHandler(NoseShapeHandler):
    
    def drawSolid(self, sketch):
        lines = sketch.sketchCurves.sketchLines

        l1 = lines.addByTwoPoints(adsk.core.Point3D.create(0,0,0), 
                                  adsk.core.Point3D.create(self._length,0,0))
        l2 = lines.addByTwoPoints(l1.endSketchPoint, 
                                  adsk.core.Point3D.create(0, self._radius, 0))
        l3 = lines.addByTwoPoints(l2.endSketchPoint, l1.startSketchPoint)
    
    def drawSolidShoulder(self, sketch):
        lines = sketch.sketchCurves.sketchLines

        l1 = lines.addByTwoPoints(adsk.core.Point3D.create(-self._shoulderLength,0,0), 
                                  adsk.core.Point3D.create(self._length,0,0))
        l2 = lines.addByTwoPoints(l1.endSketchPoint, 
                                  adsk.core.Point3D.create(0, self._radius, 0))
        l3 = lines.addByTwoPoints(l2.endSketchPoint, 
                                  adsk.core.Point3D.create(0, self._shoulderRadius, 0))
        l4 = lines.addByTwoPoints(l3.endSketchPoint, 
                                  adsk.core.Point3D.create(-self._shoulderLength, self._shoulderRadius, 0))
        l5 = lines.addByTwoPoints(l4.endSketchPoint, l1.startSketchPoint)
    
    def drawHollow(self, sketch):
        lines = sketch.sketchCurves.sketchLines

        # Calculate the offset from the end to maintain the thickness
        offset = self._length * self._thickness / self._radius

        l1 = lines.addByTwoPoints(adsk.core.Point3D.create(self._length - offset,0,0), 
                                  adsk.core.Point3D.create(self._length,0,0))
        l2 = lines.addByTwoPoints(l1.endSketchPoint, 
                                  adsk.core.Point3D.create(0, self._radius, 0))
        l3 = lines.addByTwoPoints(l2.endSketchPoint, 
                                  adsk.core.Point3D.create(0, self._radius - self._thickness, 0))
        l4 = lines.addByTwoPoints(l3.endSketchPoint, l1.startSketchPoint)
    
    def drawHollowShoulder(self, sketch):
        lines = sketch.sketchCurves.sketchLines

        # Calculate the offset from the end to maintain the thickness
        offset = self._length * self._thickness / self._radius

        l1 = lines.addByTwoPoints(adsk.core.Point3D.create(self._length - offset,0,0), 
                                  adsk.core.Point3D.create(self._length,0,0))
        l2 = lines.addByTwoPoints(l1.endSketchPoint, 
                                  adsk.core.Point3D.create(0, self._radius, 0))
        l3 = lines.addByTwoPoints(l2.endSketchPoint, 
                                  adsk.core.Point3D.create(0, self._shoulderRadius, 0))
        l4 = lines.addByTwoPoints(l3.endSketchPoint, 
                                  adsk.core.Point3D.create(-self._shoulderLength, self._shoulderRadius, 0))
        l5 = lines.addByTwoPoints(l4.endSketchPoint, 
                                  adsk.core.Point3D.create(-self._shoulderLength, self._shoulderRadius - self._thickness, 0))
        l6 = lines.addByTwoPoints(l5.endSketchPoint, 
                                  adsk.core.Point3D.create(self._thickness, self._shoulderRadius - self._thickness, 0))
        l7 = lines.addByTwoPoints(l6.endSketchPoint, 
                                  adsk.core.Point3D.create(self._thickness, self._radius - self._thickness, 0))
        l8 = lines.addByTwoPoints(l7.endSketchPoint, l1.startSketchPoint)
